<link rel="stylesheet" href="{{ url_for('static', filename='css/common-modal.css') }}">
<script src="{{ url_for('static', filename='js/common/CommonModal.js') }}"></script>

<div class="common-modal-wrapper">
  <div class="common-modal" id="common-modal">
    <div class="common-modal-splash"></div>
    <div class="common-modal-window">
      <div class="common-modal-header">
        <h3 class="common-modal-title"><!-- Modal title placehodler --></h3>
        <div class="close" title="Close window">&times;</div>
      </div>
      <div class="common-modal-content">
        <div class="common-modal-content-wrapper" id="common-modal-content-wrapper">
          <!-- Modal content placehodler -->
        </div>
      </div>
    </div>
  </div>
</div>
