{% include 'header.html' %}
<div class="container">
  <h4>Search results: (<a href="{{ url_for('search') }}">new search</a>) | <a href="{{ url_for('index') }}">Home</a></h4>
  <p>Search term: {{ search_term }} ({{ catalogue }})</p>
  {% if corrections %}
  <p>Alternate search suggestions:
    {% for cr in corrections %}
    <a href="{{ url_for('search', catalogue=catalogue, search_term=cr) }}">{{ cr }}</a> {% if not loop.last %} | {% endif %}
    {% endfor %}
  </p>
  {% endif %}

  {% for r in results %}
      {% if loop.index % 2 %}
      <div class="row">
        <div class="one-half column">
          <p class="tight"><b>{{ r.name }}</b></p>
          <p>{{ r.description }}</p>
        </div>
      {% else %}
        <div class="one-half column">
          <p class="tight"><b>{{ r.name }}</b></p>
          <p>{{ r.description }}</p>
        </div>
      </div>
      {% endif %}


  {% endfor %}
</div>
{% include 'footer.html' %}
